// java

package edu.washington.cac.myuw.servlet;

import javax.servlet.*;
import javax.servlet.http.*;
import java.lang.*;
import java.util.*;

import edu.washington.cac.gsbarnes.util.Splitter;

  /**Handles FORM input data to a servlet.
      @author GregB, FangL
      @version 2.0
    */
public class UWParameter {
    protected ServletRequest req;
  
    public UWParameter (ServletRequest req) {
        if (req == null) {
            throw new IllegalArgumentException ("UWParameter: null parameter.");
        }
  
        this.req = req;
    }
  
    public Enumeration getParameterNames() {
        return req.getParameterNames();
    }
  
    public static final int ANY = -1;
  
      /** get <n> servlet parameter values
  
        @param pName The name of the parameter
        @param n The required number of values.  ANY if any number are
        allowed
        @param allowNull If true, it's okay if the parameter does not exist
        (even if <n> != ANY)
        @param allowBlank If true, it's okay if the parameter values are blank
        @exception ParameterException 
        @return The values of the parameter
        */
    public String[] getPValues(String pName, 
  				int n, 
  				boolean allowNull,
  			     	boolean allowBlank)
      throws ParameterException 
    {
        if (pName == null ) {
  	    throw new IllegalArgumentException(
  	      "Parameter::getPValues null input");
        }
  
        String pValues[] = req.getParameterValues(pName);
          
        if (pValues == null) {
  	    if (!allowNull) {
     	        String message = "problem with parameter " + pName + ": null";
                throw new NullParameterException(message);
  	    }
  	} else {
  	  // pValues != null
  	    if ( pValues.length != n && n != ANY) {
  	        String message = "problem with parameter " + pName + 
	   	  ": length=" + pValues.length + "; expected: " + n;
  	        throw new WrongNumberOfParameterComponentsException(message);

  	    } else if (!allowBlank) {
  	        for (int i = 0; i < pValues.length; i++) {
  	            if (pValues[i].length() == 0) {
  		        String message = "value " + i + " of parameter " +
  		          pName + " is blank";
  		        throw new BlankParameterException(message);
  	            }
  	        }   
  	    } 

  	    for (int i = 0; i < pValues.length; i++) {
  	        if (pValues[i] != null &&
  		  pValues[i].length()>0 && 
  		  (pValues[i].startsWith(" ") ||
  		   pValues[i].endsWith(" "))) 
	        {
  	            pValues[i] = pValues[i].trim();
  	            //System.err.println("trim:" + pValues[i]);
  	        }
  	    }
  	}

        return pValues;
    }
    
      /** get <n> servlet parameter values.  Fail if the parameter has no 
        values, or if there are blank values
  
        @param pName The name of the parameter
  	@param n The required number of values for the parameter
        @exception ParameterException if there's a problem
        @return The values of the parameter
        */
    public String[] getPValues(String pName, int n)
      throws ParameterException 
    {
        if (pName == null ) {
  	    throw new IllegalArgumentException(
	      "Parameter::getPValues null input");
        }

        return getPValues(pName, n, false, false);
    }
  	    
      /** get all the values for a servlet parameter, which must exist, but
  	  can be blank.
  
          @param pName The name of the parameter
          @exception ParameterException if there's a problem
          @return The values of the parameter
         */
    public String[] getPValues(String pName)
      throws ParameterException 
    {
        if (pName == null ) {
  	    throw new IllegalArgumentException(
	      "Parameter::getPValues null input");
        }

        return getPValues(pName, ANY, false, true);
    }
    
      /** get one servlet parameter value, which must exist, must be unique, 
          and must not be blank
  
          @param pName The name of the parameter
          @exception ParameterException 
          @return The value of the parameter
  	*/
    public String getPValue(String pName)
      throws ParameterException 
    {
        if (pName == null ) {
  	    throw new IllegalArgumentException(
	      "Parameter::getPValue null input");
        }
  
        return getPValues(pName, 1, false, false)[0];
    }
  
      /** get the first value of a servlet parameter
      
          @param pName The name of the parameter
  	  @param strict If true, equivalent to getPValue(String pName).  
  	    Otherwise, nulls and blanks are allowed, and it's okay if the
  	    parameter has more than one value
          @exception ParameterException
          @return The value of the parameter, or null if it doesn't exist
         */
    public String getPValue(String pName, 
  			boolean strict) 
      throws ParameterException 
    {
        if (pName == null ) {
  	    throw new IllegalArgumentException(
  	      "Parameter::getPValue null input");
        }
  
  	String pValues[] = getPValues(pName, strict ? 1 : ANY, !strict, 
  	  !strict);
  
  	if (pValues == null) {
  	    return null;
  	} else {
  	    return pValues[0];
  	}
    }
  
      /** given a parameter value with components separated by <separator>,
        return the components as an array of strings.
  
        @exception throw a ParameterException if the number
          of components generated is not equal to <expected>.
       */
    public String[] paramSplit2(String s, 
			        String separator,
  				int expected) 
      throws ParameterException 
    {
  	if (s == null ) {
  	    throw new IllegalArgumentException("paramSplit2: Null input.");
  	}
  
//System.err.println("paramSplit: " + s + " ==> " + expected);
  	StringTokenizer t = new StringTokenizer(s, separator);
  	String[] fields = new String[expected];
  	int i =0;

  	while (t.hasMoreTokens()) {
  	    if ( i > expected ) {
  	        throw new TooManyParameterComponentsException(
  	          "Has more components than expected: " + s);
  	    }
  	    
  	    fields[i] = t.nextToken();
  	    i++;
  	}
  
  	if (i < expected) {
  	    throw new TooFewParameterComponentsException(
  	      "Has fewer components than expected: " + s);
  	}
  
  	return fields;
    }
  
        /** given a parameter value with components separated by <separator>,
           return the components as an array of strings.
  
           @exception throw a ParameterException if the number
           of components generated is not equal to <expected>.
          */
    public String[] paramSplit(String s, 
			        String separator,
  				int expected) 
      throws ParameterException 
    {
        if (s == null ) {
            throw new IllegalArgumentException(
  	      "Parameter::paramSplit: Null parameter.");
        }
  
//System.err.println("paramSplit: " + s + " ==> " + expected);
      	
        Splitter spl = new Splitter(separator);
        String[] sa = spl.split(s);
      	
        if (sa.length != expected) {
  	    throw new WrongNumberOfParameterComponentsException(
  	      "expected " + expected + " components, found " + sa.length);
  	}
  	
  	return sa;
    }
  
    /** print all the Form parameters 
     */
    public void printAll() {
        Enumeration e = req.getParameterNames();
  
        while (e.hasMoreElements()) {
            String pName = (String) e.nextElement();
            System.err.print(pName + ": ");
            String pValues[] = req.getParameterValues(pName);
  
            for (int i =0; i<pValues.length; i++) {
  	        if (i>0) {
  	            System.err.print(", ");
  		}
  
  	        System.err.print(pValues[i]);
            }
  
            System.err.println("");
        }
  
        System.err.println("--end--");
    }
}
