// java

package pubevents;

import javax.servlet.http.HttpServletRequest;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.myuw.servlet.ParameterException;
import edu.washington.cac.myuw.servlet.UWParameter;

/** class to handle parameters for a Public Events Calendar
    @author Leman Chung, Greg Barnes
    @version 2.1
  */
public class PEParameter extends CalParameter 
{
  /** Create a <code>PEParameter</code> from an HTTP request
    @param req The HTTP request object
   */
  public PEParameter(HttpServletRequest req) 
  {
    super(req);
  }

  /** Name of the parameter that indicates an area code */
  public static final String AREA_CODE = "areaCode";

  /** Name of the parameter that indicates the first 3 numbers in a phone # */
  public static final String PHONE_BEGIN = "phoneBegin";

  /** Name of the parameter that indicates the last 4 numbers in a phone # */
  public static final String PHONE_END = "phoneEnd";

  /** Name of the parameter that indicates info after a phone # */
  public static final String MORE_PHONE = "morePhone";

  /** Name of the parameter that indicates the value of a submit button */
  public static final String SUBMIT = "submit";

  /** Name of the parameter that indicates whether this is an alert */
  public static final String RED_ALERT = "redAlert";

  /** Name of the parameter that indicates the creator of an object */
  public static final String CREATOR = "creator";

  /** Name of the parameter that indicates a search string */
  public static final String SEARCH_STRING = "searchString";

  /** Name of the parameter that indicates whether a search should include
      old events */
  public static final String INCLUDE_OLD_EVENTS = "includeOldEvents";

  /** Name of the parameter that indicates a number of eventids*/
  public static final String EVENT_IDS = "eventids";

  /** Name of the parameter that indicates the userid of a user */
  public static final String USERID = "userid";

  /** Name of the parameter that indicates the authorization type of a user */
  public static final String USERTYPE = "usertype";

  /** Name of the parameter that indicates the first name of a user */
  public static final String FIRSTNAME = "firstname";

  /** Name of the parameter that indicates the last name of a user */
  public static final String LASTNAME = "lastname";

  /** Name of the parameter that indicates the phone number of a user */
  public static final String UPHONE = "uphone";

  /** Name of the parameter that indicates the email of a user */
  public static final String UEMAIL = "uemail";

  /** Name of the parameter that indicates the department of a user */
  public static final String DEPARTMENT = "department";

  /** Name of the parameter that indicates a hidden keyword */
  public static final String HIDDEN_KEYWORD = "hiddenkeyword";

//%%%% The following should go away when the forms are fixed so that
//%%%% you can't enter multiple entities at the same time
  /** Name of the parameter for a sponsor link */
  public static final String SLINK = "sponsorUrl";
  /** Name of the parameter for a location link */
  public static final String LLINK = "locationUrl";
//%%%% I'm dubious this is necessary
  public static final String SEQ = "seq";

  /**
    Test if the <code>RED_ALERT</code> parameter has a value,
    @return true if the <code>RED_ALERT</code> parameter has a value,
      false otherwise
   */
  public boolean getRedAlert()
  {
    return getCheckValue(RED_ALERT);
  }

  /**
    Test if the <code>INCLUDE_OLD_EVENTS</code> parameter has a value,
    @return true if the <code>INCLUDE_OLD_EVENTS</code> parameter has a value,
      false otherwise
   */
  public boolean getIncludeOldEvents()
  {
    return getCheckValue(INCLUDE_OLD_EVENTS);
  }

  /** 
      Get the value of the <code>SEARCH_STRING</code> parameter
      @return the value of the <code>SEARCH_STRING</code> parameter
    */
  public String getSearchString()
  {
    return getString(SEARCH_STRING);
  }

  /** 
      Get the value of the <code>USERID</code> parameter
      @return the value of the <code>USERID</code> parameter
    */
  public String getUserid()
  {
    return getString(USERID);
  }

  /** 
      Get the value of the <code>DEPARTMENT</code> parameter
      @return the value of the <code>DEPARTMENT</code> parameter
    */
  public String getDepartment()
  {
    return getString(DEPARTMENT);
  }

  /** @return Is a string made up entirely of digits? */
  private boolean allDigits(String s)
  {
    for (int i = 0; i < s.length(); i++) {
      if (!Character.isDigit(s.charAt(i))) {
        return false;
      }
    }

    return true;
  }

  /**
    Check if a String is composed only of digits and is the proper length
    @param s The String in question
    @param expectedLength The length the string should be
    @exception NumberFormatException if the String is the wrong length,
       or contains non-digits
   */
  private void checkDigits(String s, int expectedLength) 
      throws NumberFormatException
  {
    if (s == null || s.length() != expectedLength) {
      throw new NumberFormatException("Wrong number of digits: " + s);
    } else if (!allDigits(s)) {
      throw new NumberFormatException("Not a numeric value: " + s);
    }
  }

  /** 
      Get the value of the <code>PHONE</code> parameter
      @return the value of the <code>PHONE</code> parameter
    */
  public String getPhone() throws NumberFormatException
  {
    if (super.getPhone() != null) {
      return super.getPhone();
    } else {
      checkDigits(getString(AREA_CODE), 3);
      checkDigits(getString(PHONE_BEGIN), 3);
      checkDigits(getString(PHONE_END), 4);

      return getString(AREA_CODE) + "-" + getString(PHONE_BEGIN) + "-" +
             getString(PHONE_END) + 
             ((getString(MORE_PHONE) == null) ? "" : 
                                                " " + getString(MORE_PHONE));
    }
  }

  /**
    @return true for the <code>PUBLIC</code> value, since this is public event.
   */
  public boolean getPublic()
  {
    return true;
  }

  /** 
      Get the value of the <code>UEMAIL</code> parameter
      @return the value of the <code>UEMAIL</code> parameter
    */
  public String getUemail()
  {
    return getString(UEMAIL);
  }

  /** 
      Get the value of the <code>UPHONE</code> parameter
      @return the value of the <code>UPHONE</code> parameter
    */
  public String getUphone()
  {
    return getString(UPHONE);
  }

  /** 
      Get the value of the <code>LASTNAME</code> parameter
      @return the value of the <code>LASTNAME</code> parameter
    */
  public String getLastname()
  {
    return getString(LASTNAME);
  }

  /** 
      Get the value of the <code>FIRSTNAME</code> parameter
      @return the value of the <code>FIRSTNAME</code> parameter
    */
  public String getFirstname()
  {
    return getString(FIRSTNAME);
  }

  /** 
      Get the value of the <code>CREATOR</code> parameter
      @return the value of the <code>CREATOR</code> parameter
    */
  public String getCreator()
  {
    return getString(CREATOR);
  }
  /** 
      Get the value of the <code>SLINK</code> parameter
      @return the value of the <code>SLINK</code> parameter
    */
  public String getSlink()
  {
    return getUrl(SLINK);
  }

  /** 
      Get the value of the <code>LLINK</code> parameter
      @return the value of the <code>LLINK</code> parameter
    */
  public String getLlink()
  {
    return getUrl(LLINK);
  }

  /** 
      Get the value of the <code>USERTYPE</code> parameter
      @return the value of the <code>USERTYPE</code> parameter
    */
  public String getUsertype() {
    return getString(USERTYPE);
  }

  /** 
      Get the value of the <code>SUBMIT</code> parameter
      @return the value of the <code>SUBMIT</code> parameter
    */
  public String getSubmit() {
    return getString(SUBMIT);
  }

  /** 
      Get the value of the <code>HIDDEN_KEYWORD</code> parameter
      @return the value of the <code>HIDDEN_KEYWORD</code> parameter
    */
  public String getHiddenKeyword() {
    return getString(HIDDEN_KEYWORD);
  }

  /** 
      Get the values of the <code>EVENT_IDS</code> parameter
      @return the values of the <code>EVENT_IDS</code> parameter
    */
  public int[] getEventIds() {
    String[] stringIds;

    try {
      stringIds = getPValues(EVENT_IDS, UWParameter.ANY, true, false);
    } catch (ParameterException e) {
      return null;
    }

    if (stringIds == null) {
      return null;
    }

    int[] intIds = new int[stringIds.length];

    for (int i = 0; i < stringIds.length; i++) {
      intIds[i] = Integer.parseInt(stringIds[i]);
    }

    return intIds;
  }
}
