// java

package uwcal.out;

import java.sql.SQLException;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.ItemException;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.EventsI;
import edu.washington.cac.calendar.data.User;

/** An object that creates a list of events for many consecutive days

    @author Greg Barnes
    @version 1.0
  */
public class MultidayEventList extends EventList 
{
  /** Create a new instance
    @param ep Object that prints a single event
   */
  protected MultidayEventList(EventPrinter ep)
  {
    super(ep);
  }

  /**
    Are we currently printing the first day?
    @return Are we currently printing the first day?
   */
  private boolean currentDayIsFirstDay()
  {
    return this.ep.getDate().isSameDay(this.ep.getFirstDay());
  }

  /**
    Is the current day the same as an event's start day?
    @param e The event
    @return Is the current day the same as an event's start day?
   */
  private boolean currentDayIsStartDay(Event e)
  {
    return this.ep.getDate().isSameDay(e.startCalendar());
  }

  /**
    Should we print this event?
    @param e Event
    @return Should we print this event?
   */
  private boolean shouldPrintEvent(Event e)
  {
    return currentDayIsFirstDay() || currentDayIsStartDay(e);
  }

  /**
    Get a row that gives the current date
    @return a row that gives the current date
   */
  private Element dateHeader()
  {
    return new TR().addElement(new TD().
        addElement(new BR()).
        addElement(Util.linkItem(Dayview.url(ep.getDate()),
                                 ep.getDate().dateString())).
        addElement("\n").
        setClass(Style.LIST_DATE_HEADER));
  }

  /**
    Get The list of events as HTML
    @return The list of events as HTML
    @param ea Ordered list of events
    @param es EventsI cache
   */
  protected Element eventsRows(Event[] ea, EventsI es) 
  {
    int i;
    ElementContainer ec = new ElementContainer();
    boolean eventPrinted = false;

    for (i = 0; i < ea.length; i++) {
      if (shouldPrintEvent(ea[i])) {
        if (!eventPrinted) {
          eventPrinted = true;

          /* Print a date header before the first event we print, unless 
             this is one of the events
             that started before the time period in question
           */
          if (!ea[i].startCalendar().isEarlierDay(ep.getFirstDay())) 
          {
            ec.addElement(dateHeader());
          }
        }

        /* add a date header between the events that start before the first
           day and the events that start on the first day
         */
        if (i > 0 && ea[i].startCalendar().isSameDay(ep.getFirstDay()) &&
            !ea[i].startCalendar().isSameDay(ea[i-1].startCalendar())) 
        {
          ec.addElement(dateHeader());
        }

        ec.addElement(new TR().
                      addElement(ep.format(ea[i], es)).
                      addElement("\n"));
      }
    }

    return ec;
  }

  /**
    Does the list generated from a set of events contain any events?
    @param es The set of events
    @return Does the list generated from a set of events contain any events?
    @exception SQLException If we have to go to the database, and
      there is a problem
    @exception ItemException If we have to go to the database, 
      and it contains bad data
   */
  private boolean hasEvents(EventsI es)
      throws SQLException, ItemException
  {
    for (MyCalendar current = ep.getFirstDay();
         !ep.getLastDay().isEarlierDay(current);
         current = current.tomorrow())
    {
      if (es.oneDaysEvents(current).length > 0) {
        return true;
      }
    }

    return false;
  }

  /**
    get a list of all appropriate events
    @param es A cache of the events
    @return The list, as HTML
    @exception SQLException If we have to go to the database, and
      there is a problem
    @exception ItemException If we have to go to the database, 
      and it contains bad data
   */
  protected Element allEvents(EventsI es) throws SQLException, ItemException
  {
    if (!hasEvents(es)) {
      return new TR().
             addElement(new TD().
                        addElement(Entities.NBSP +
                                   DisplayText.NO_EVENTS).
                        setClass(Style.EVENTLIST_DESC));
    } else {
      return super.allEvents(es);
    }
  }
}
