// java

package uwcal.out.forms;

import org.apache.ecs.AlignType;
import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.Entities;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.B;
import org.apache.ecs.html.Input;
import org.apache.ecs.html.TD;
import org.apache.ecs.html.TextArea;
import org.apache.ecs.html.TR;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.data.CalendarObject;
import edu.washington.cac.calendar.data.LinkEntity;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.calendar.data.LongdescEntity;
import edu.washington.cac.calendar.data.Sponsor;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.DisplayText;
import uwcal.out.Style;
import uwcal.out.Util;

/** Common form elements 
   @author Greg Barnes
   @version 1.1
 */
public class CommonFormElements 
{
  CalendarObject c;

  /**
    Constructor used when adding a new object
   */
  public CommonFormElements() 
  { 
    c = null;
  }

  /**
    Constructor used when editing an old object
   */
  CommonFormElements(CalendarObject c) 
  {
    this.c = c;
  }

  protected static TD standardTD() 
  {
    return new TD().setAlign(AlignType.LEFT);
  }

  /**
    Get a table row with two cells
    @param cell1 first cell
    @param cell2 second cell
    @return The table row
   */
  public static TR tableRowTwoCells(Element cell1, 
                                       Element cell2) 
  {
    if (!(cell1 instanceof TD)) {
      cell1 = standardTD().addElement(cell1);
    }

    if (!(cell2 instanceof TD)) {
      cell2 = standardTD().addElement(cell2);
    }

    return new TR().addElement(cell1).addElement(cell2);
  }

  public static TR tableRowOneCell(Element cell) 
  {
    TD td;

    if (cell instanceof TD) {
        td = (TD) cell;
    } else {
        td = standardTD().addElement(cell);
    }

    return new TR().addElement(td);
  }

  protected static Element centeredBoldText(String text) 
  {
    return tableRowOneCell(new TD().setAlign(AlignType.CENTER).
        setColSpan(2).addElement(new B().addElement(text)));
  }

  /**
    Get a header in the form
    @param text Text for the heading
    @param cl stylesheet class for the row, null if the class should not be
           set
    @return the header
   */
  public static Element headerRow(String text, String cl) 
  {
    TD td = new TD().setColSpan(2).addElement(text);

    if (cl != null) {
      td.setClass(cl);
    }

    return tableRowOneCell(td);
  }

  /**
    Get a blank table row
    @param cl stylesheet class for the row, null if the class should not be
           set
    @return a blank table row
   */
  public static Element blankRow(String cl)
  {
    return headerRow(Entities.NBSP, cl);
  }

  /**
    Get a blank table row
    @return a blank table row
   */
  public static Element blankRow()
  {
    return blankRow(null);
  }

  public static Element oldNewInput(Element oldName, 
                                    Element oldValue, 
                                    Element newName, 
                                    Element newInput)
  {
    return new ElementContainer().
        addElement(tableRowTwoCells(oldName, oldValue)).
        addElement("\n").
        addElement(tableRowTwoCells(newName, newInput));
  }

  /**
    Get a cell prompting for input
    @param name Text next to the input item
    @param input Input element (textbox, checkbox, menu, etc.)
    @return a cell prompting for input
   */
  public static Element newInput(Element name, Element input)
  {
    return tableRowOneCell(new ElementContainer().
                           addElement(name).
                           addElement(" ").
                           addElement(input));
  }

  private Element linkInput(String initialValue) 
  {
    return new Input(Input.TEXT, CalParameter.LINK, initialValue).setSize(25);
  }

  private String linkAsText(LinkEntity le) 
  {
    return le.getLink() == null ? DisplayText.NO_VALUE : le.getLink();
  }

  private String linkAsEditableText(LinkEntity le) 
  {
    return le.getLink() == null ?  FormsDisplayText.INITIAL_LINK : le.getLink();
  }

  Element link(String description) 
  {
    if (c == null) {
      return newInput(Util.spanItem(description, Style.STD_TEXT), 
                      linkInput(FormsDisplayText.INITIAL_LINK));
    } else {
      return oldNewInput(FormsDisplayText.EDIT_OLD_LINK,
                         new StringElement(linkAsText((LinkEntity) c)),
                         FormsDisplayText.EDIT_NEW_LINK, 
                         linkInput(linkAsEditableText((LinkEntity) c)));
    }
  }

  protected Element publicPrivateInput(boolean isPublic) 
  {
    return new Input(Input.CHECKBOX, CalParameter.PUBLIC, "").
           setChecked(isPublic);
  }

  protected Element publicPrivate() 
  {
    ElementContainer ec = new ElementContainer();

    if (c == null) {      // adding new calendar object
      ec.addElement(newInput(
          Util.spanItem(FormsDisplayText.PUBLICF, Style.STD_TEXT),
          publicPrivateInput(false)));
    } else {                // editing old calendar object
      ec.addElement(oldNewInput(FormsDisplayText.EDIT_OLD_PUBLIC,
                                new StringElement(c.isPublic() + ""),
                                new ElementContainer().addElement(new B()).
                                    addElement(FormsDisplayText.PUBLICF),
                                    publicPrivateInput(c.isPublic())));
    }

    return ec.addElement("\n").
        addElement(tableRowTwoCells(new ElementContainer(), 
                                    FormsDisplayText.PUBLIC_ADVISORY));
  }

  protected Element longdescInput(String initialValue) 
  {
    return new TextArea(5, 40).setName(CalParameter.LONG_DESCRIPTION).
           addElement(initialValue);
  }

  protected Element longdesc() 
  {
    if (c == null) {
      return newInput(
          Util.spanItem(FormsDisplayText.LONG_DESCRIPTION, Style.STD_TEXT), 
          longdescInput(""));
    } else {
      return oldNewInput(FormsDisplayText.EDIT_OLD_LONG_DESCRIPTION,
            new StringElement(
                Coder.HTMLencode(((LongdescEntity) c).getLongdesc())),
            FormsDisplayText.EDIT_NEW_LONG_DESCRIPTION, 
            longdescInput(
                Coder.HTMLencode(((LongdescEntity) c).getLongdesc())));
    }
  }

  protected Element shortdescInput(String initialValue) 
  {
    return new Input(Input.TEXT, CalParameter.SHORT_DESCRIPTION, 
        initialValue).setSize(25);
  }

  protected static Element required() 
  {
    return Util.spanItem("*", Style.REQUIRED_FIELD, 
        FormsDisplayText.REQUIRED);
  }

  protected static Element requiredFieldName(Element name) 
  {
    return new ElementContainer().
        addElement(name).addElement(required());
  }

  protected static Element requiredFieldName(String name) 
  {
    return requiredFieldName(Util.spanItem(name + " ", Style.STD_TEXT));
  }

  protected Element shortdesc() 
  {
    if (c == null) {
      return newInput(requiredFieldName(FormsDisplayText.NAME),
          shortdescInput(""));
    } else {
      return oldNewInput(FormsDisplayText.EDIT_OLD_NAME,
          new StringElement(
              Coder.HTMLencode(((LongdescEntity) c).getShortdesc())),
          requiredFieldName(FormsDisplayText.EDIT_NEW_NAME),
          shortdescInput(
              Coder.HTMLencode(((LongdescEntity) c).getShortdesc())));
    }
  }

  /**
    Get HTML for a submit button
    @param buttonText text for the submit button
    @return HTML for the submit button
   */
  protected static Element submitButton(String buttonText)
  {
    return new Input().setName(CalParameter.SUBMIT).
           setType(Input.SUBMIT).setValue(buttonText);
  }

  /**
    Get HTML for a submit button row
    @param buttonText text for the submit button
    @return HTML for the submit button row
   */
  protected static Element submitButtonRow(String buttonText)
  {
    return new TR().
           addElement(new TD().
                      addElement(submitButton(buttonText)).
                      setID(Style.SUBMIT_CELL));
  }
}
